<?php

namespace Nest\Types\Definitions;

use Nest\Types\Exceptions\InvalidSerializedValueException;
use Nest\Types\Exceptions\InvalidStoredValueException;
use Nest\Types\Exceptions\ValueNotSerializableException;
use Nest\Types\Exceptions\ValueNotStorableException;
use Nest\Types\SerializableType;
use Nest\Types\SqlType;
use Nest\Types\Type;
use DateTime;
use DateTimeInterface;
use Override;

class DateTimeType extends Type implements SerializableType, SqlType
{
	/**
	 * @inheritDoc
	 * @throws ValueNotSerializableException
	 */
	#[Override] public function serialize(mixed $value): string
	{
		if (!($value instanceof DateTimeInterface))
			throw new ValueNotSerializableException($this, $value);

		return $value->format(DateTimeInterface::ATOM);
	}

	/**
	 * @inheritDoc
	 * @throws InvalidSerializedValueException
	 */
	#[Override] public function deserialize(float|object|int|array|string $value): DateTime
	{
		if (!is_string($value))
			throw new InvalidSerializedValueException($this, $value);

		return DateTime::createFromFormat(DateTimeInterface::ATOM, $value);
	}


	/**
	 * SQL datetime format.
	 */
	public const string SQL_FORMAT = "Y-m-d H:i:sO";

	/**
	 * @inheritDoc
	 * @throws ValueNotStorableException
	 */
	#[Override] public function toSql(mixed $value): string
	{
		if (!($value instanceof DateTimeInterface))
			throw new ValueNotStorableException($this, $value);

		return $value->format(static::SQL_FORMAT);
	}

	/**
	 * @inheritDoc
	 * @throws InvalidStoredValueException
	 */
	#[Override] public function fromSql(array|int|string $value): DateTime
	{
		if (is_array($value))
			throw new InvalidStoredValueException($this, $value);

		if (is_int($value))
			// If it's an integer, we suppose that it's a unix timestamp.
			DateTime::createFromFormat("U", "$value");

		return DateTime::createFromFormat(static::SQL_FORMAT, $value);
	}
}
